import { IGNORED_GLOBAL_BINARIES } from '../constants.js';
import parsedArgs from '../util/parseArgs.js';
import { timerify } from '../util/performance.js';
import { getBinariesFromScripts, getPackageManifest } from './helpers.js';
const { values: { production: isProduction = false }, } = parsedArgs;
const findManifestDependencies = async (ignoreBinaries, manifest, isRoot, dir, cwd) => {
    const scriptFilter = isProduction ? ['start'] : [];
    const referencedDependencies = new Set();
    const peerDependencies = new Map();
    const scripts = Object.entries(manifest.scripts ?? {}).reduce((scripts, [scriptName, script]) => {
        if (script && (scriptFilter.length === 0 || (scriptFilter.length > 0 && scriptFilter.includes(scriptName)))) {
            return [...scripts, script];
        }
        return scripts;
    }, []);
    const referencedBinaries = getBinariesFromScripts(scripts)
        .filter(binaryName => !IGNORED_GLOBAL_BINARIES.includes(binaryName))
        .filter(binaryName => !ignoreBinaries.includes(binaryName));
    const installedBinaries = new Map();
    const deps = [...Object.keys(manifest.dependencies ?? {}), ...Object.keys(manifest.devDependencies ?? {})];
    for (const packageName of deps) {
        const manifest = await getPackageManifest(dir, packageName, isRoot, cwd);
        if (manifest) {
            const binaries = typeof manifest.bin === 'string' ? [packageName] : Object.keys(manifest.bin ?? {});
            binaries.forEach(binaryName => {
                if (installedBinaries.has(binaryName)) {
                    installedBinaries.get(binaryName)?.add(packageName);
                }
                else {
                    installedBinaries.set(binaryName, new Set([packageName]));
                }
            });
            const packagePeerDependencies = Object.keys(manifest.peerDependencies ?? {});
            packagePeerDependencies.forEach(packagePeerDependency => {
                if (peerDependencies.has(packagePeerDependency)) {
                    peerDependencies.get(packagePeerDependency)?.add(packageName);
                }
                else {
                    peerDependencies.set(packagePeerDependency, new Set([packageName]));
                }
            });
        }
    }
    for (const binaryName of referencedBinaries) {
        if (installedBinaries.has(binaryName)) {
            installedBinaries.get(binaryName)?.forEach(packageName => referencedDependencies.add(packageName ?? binaryName));
        }
        else {
            referencedDependencies.add(binaryName);
        }
    }
    return {
        dependencies: Array.from(referencedDependencies),
        peerDependencies,
        installedBinaries,
    };
};
export const findDependencies = timerify(findManifestDependencies);
