import { ESLint } from 'eslint';
import { compact } from '../../util/array.js';
import { _firstGlob } from '../../util/glob.js';
import load from '../../util/loader.js';
import { getPackageName } from '../../util/modules.js';
import { timerify } from '../../util/performance.js';
import { resolvePluginPackageName, customResolvePluginPackageNames, getDependenciesFromSettings } from './helpers.js';
export const isEnabled = ({ dependencies }) => dependencies.has('eslint');
export const CONFIG_FILE_PATTERNS = ['.eslintrc', '.eslintrc.{js,json}', 'package.json'];
export const ENTRY_FILE_PATTERNS = ['eslint.config.js'];
const findESLintDependencies = async (configFilePath, { cwd, manifest, workspaceConfig }) => {
    if (configFilePath.endsWith('package.json') && !manifest.eslintConfig)
        return [];
    const config = configFilePath.endsWith('package.json')
        ? manifest.eslintConfig
        : await load(configFilePath);
    const rootExtends = config?.extends ? [config.extends].flat().map(customResolvePluginPackageNames) : [];
    const patterns = [workspaceConfig.entryFiles, ...(config?.overrides?.map(overrides => overrides.files) ?? [])];
    const samples = await Promise.all(patterns.map(patterns => _firstGlob({ patterns, cwd })));
    const sampleFilePaths = samples
        .filter((filePath) => typeof filePath !== 'undefined')
        .map(String);
    const engine = new ESLint({ cwd, overrideConfigFile: configFilePath, useEslintrc: false });
    const calculateConfigForFile = async (sampleFile) => await engine.calculateConfigForFile(sampleFile);
    const dependencies = await Promise.all(sampleFilePaths.map(calculateConfigForFile)).then(configs => configs.flatMap(config => {
        if (!config)
            return [];
        const plugins = config.plugins?.map(resolvePluginPackageName) ?? [];
        const parsers = config.parser ? [config.parser] : [];
        const extraParsers = config.parserOptions?.babelOptions?.presets ?? [];
        const settings = config.settings ? getDependenciesFromSettings(config.settings) : [];
        return [...parsers, ...extraParsers, ...plugins, ...settings].map(getPackageName);
    }));
    return compact([...rootExtends, ...dependencies.flat()]);
};
export const findDependencies = timerify(findESLintDependencies);
