import path from 'node:path';
import { OwnershipEngine } from '@snyk/github-codeowners/dist/lib/ownership/index.js';
import { relative } from '../util/path.js';
import { ISSUE_TYPE_TITLE } from './constants.js';
const logIssueLine = (owner, filePath, symbols, parentSymbol) => {
    const symbol = symbols ? `: ${symbols.join(', ')}` : '';
    const parent = parentSymbol ? ` (${parentSymbol})` : '';
    console.log(`${owner} ${relative(filePath)}${symbol}${parent}`);
};
const logIssueSet = (issues, title) => {
    title && console.log(`--- ${title} (${issues.length})`);
    if (issues.length) {
        issues
            .sort((a, b) => (a.owner < b.owner ? -1 : 1))
            .forEach(issue => console.log(issue.owner, issue.symbol.startsWith('/') ? relative(issue.symbol) : issue.symbol));
    }
    else {
        console.log('Not found');
    }
};
const logIssueRecord = (issues, title) => {
    title && console.log(`--- ${title} (${issues.length})`);
    if (issues.length) {
        const sortedByFilePath = issues.sort((a, b) => (a.owner < b.owner ? -1 : 1));
        sortedByFilePath.forEach(({ filePath, symbols, owner, parentSymbol }) => logIssueLine(owner, filePath, symbols, parentSymbol));
    }
    else {
        console.log('Not found');
    }
};
export default ({ report, issues, options }) => {
    let opts = {};
    try {
        opts = options ? JSON.parse(options) : opts;
    }
    catch (error) {
        console.error(error);
    }
    const codeownersFilePath = path.resolve(opts.path ?? '.github/CODEOWNERS');
    const codeownersEngine = OwnershipEngine.FromCodeownersFile(codeownersFilePath);
    const reportMultipleGroups = Object.values(report).filter(Boolean).length > 1;
    const [dependenciesOwner = '[no-owner]'] = codeownersEngine.calcFileOwnership('package.json');
    const fallbackOwner = dependenciesOwner;
    const calcFileOwnership = (filePath) => codeownersEngine.calcFileOwnership(relative(filePath))[0] ?? fallbackOwner;
    const addOwner = (issue) => ({ ...issue, owner: calcFileOwnership(issue.filePath) });
    for (const [type, isReportType] of Object.entries(report)) {
        if (isReportType) {
            const title = reportMultipleGroups && ISSUE_TYPE_TITLE[type];
            if (issues[type] instanceof Set) {
                const toIssue = (filePath) => ({ type, filePath, symbol: filePath });
                const issuesForType = Array.from(issues[type]).map(toIssue);
                logIssueSet(issuesForType.map(addOwner), title);
            }
            else if (type === 'duplicates') {
                const issuesForType = Object.values(issues[type]).map(Object.values).flat().map(addOwner);
                logIssueRecord(issuesForType, title);
            }
            else {
                const issuesForType = Object.values(issues[type]).map(issues => {
                    const items = Object.values(issues);
                    return addOwner({ ...items[0], symbols: items.map(issue => issue.symbol) });
                });
                logIssueRecord(issuesForType, title);
            }
        }
    }
};
