import path from 'node:path';
import fg from 'fast-glob';
import { globby } from 'globby';
import { compact } from './array.js';
import { debugLogObject } from './debug.js';
import { timerify } from './performance.js';
const ensurePosixPath = (value) => value.split(path.sep).join(path.posix.sep);
const prependDirToPattern = (workingDir, pattern) => {
    if (pattern.startsWith('!'))
        return '!' + path.posix.join(workingDir, pattern.slice(1));
    return path.posix.join(workingDir, pattern);
};
export const negate = (pattern) => `!${pattern}`;
export const hasProductionSuffix = (pattern) => pattern.endsWith('!');
export const hasNoProductionSuffix = (pattern) => !pattern.endsWith('!');
const removeProductionSuffix = (pattern) => pattern.replace(/!$/, '');
const glob = async ({ cwd, workingDir = cwd, patterns, ignore = [], gitignore = true, }) => {
    const cwdPosix = ensurePosixPath(cwd);
    const workingDirPosix = ensurePosixPath(workingDir);
    const relativePath = path.posix.relative(cwdPosix, workingDirPosix);
    const prepend = (pattern) => prependDirToPattern(relativePath, pattern);
    const globPatterns = compact([patterns].flat().map(prepend).map(removeProductionSuffix));
    const ignorePatterns = [...ignore, '**/node_modules/**'];
    debugLogObject(2, "Globbin'", { cwd, globPatterns, ignorePatterns });
    return globby(globPatterns, {
        cwd,
        ignore: ignorePatterns,
        gitignore,
        absolute: true,
        dot: true,
    });
};
const pureGlob = async ({ cwd, patterns, ignore }) => globby(patterns, {
    cwd,
    ignore: [...ignore, '**/node_modules/**'],
    absolute: true,
});
const dirGlob = async ({ cwd, patterns, ignore }) => globby(patterns, {
    cwd,
    ignore: [...ignore, '**/node_modules/**'],
    expandDirectories: false,
    onlyDirectories: true,
});
const firstGlob = async ({ cwd, patterns }) => {
    const stream = fg.stream(patterns.map(removeProductionSuffix), { cwd });
    for await (const entry of stream) {
        return entry;
    }
};
export const _glob = timerify(glob);
export const _pureGlob = timerify(pureGlob);
export const _dirGlob = timerify(dirGlob);
export const _firstGlob = timerify(firstGlob);
